@@ -10,7 +10,7 @@
 #' @param wait In seconds, defaults to 1 due to time-outs with frequent queries. Time to wait between queries to the APIs. If data are cached locally, wait time is not applied. If you are running many queries systematically you may want to add some waiting time between queries.
 #' @param attempts Defaults to 5. Number of times it re-attempts to reach the API before failing.
 #'
-#' @return A data frame (a tibble) with wight columns: `source_title_url`, `source_wikipedia_title`, `source_qid`, `wikipedia_title`, `wikipedia_id`, `qid`, `description`, and `language`.
+#' @return A data frame (a tibble) with eight columns: `source_title_url`, `source_wikipedia_title`, `source_qid`, `wikipedia_title`, `wikipedia_id`, `qid`, `description`, and `language`.
 #' @export
 #'
 #' @examples
@@ -77,14 +77,13 @@ tw_get_wikipedia_page_links <- function(url = NULL,
     }
   )
 
-  if (disconnect_db == TRUE) {
-    tw_disconnect_from_cache(
-      cache = TRUE,
-      cache_connection = db,
-      disconnect_db = disconnect_db,
-      language = language
-    )
-  }
+  tw_disconnect_from_cache(
+    cache = cache,
+    cache_connection = db,
+    disconnect_db = disconnect_db,
+    language = language
+  )
+
   wikipedia_page_links_df
 }
 
@@ -119,15 +118,28 @@ tw_get_wikipedia_page_links_single <- function(url = NULL,
                                                wait = 1,
                                                attempts = 5,
                                                wikipedia_page_qid_df = NULL) {
+  db <- tw_connect_to_cache(
+    connection = cache_connection,
+    language = language,
+    cache = cache
+  )
+
   if (tw_check_cache(cache) == TRUE & overwrite_cache == FALSE) {
     db_result <- tw_get_cached_wikipedia_page_links(
       title = title,
       language = language,
       cache = cache,
-      cache_connection = cache_connection,
-      disconnect_db = disconnect_db
+      cache_connection = db,
+      disconnect_db = FALSE
     )
     if (is.data.frame(db_result) & nrow(db_result) > 0) {
+      tw_disconnect_from_cache(
+        cache = cache,
+        cache_connection = db,
+        disconnect_db = disconnect_db,
+        language = language
+      )
+
       return(db_result %>%
         dplyr::collect())
     }
@@ -291,21 +303,19 @@ tw_get_wikipedia_page_links_single <- function(url = NULL,
   if (tw_check_cache(cache) == TRUE) {
     tw_write_wikipedia_page_links_to_cache(
       df = output_linked_df,
-      cache_connection = cache_connection,
+      cache_connection = db,
       language = language,
       overwrite_cache = overwrite_cache,
       disconnect_db = disconnect_db
     )
   }
 
-  if (disconnect_db == TRUE) {
-    tw_disconnect_from_cache(
-      cache = TRUE,
-      cache_connection = cache_connection,
-      disconnect_db = disconnect_db,
-      language = language
-    )
-  }
+  tw_disconnect_from_cache(
+    cache = cache,
+    cache_connection = db,
+    disconnect_db = disconnect_db,
+    language = language
+  )
 
   output_linked_df
 }
@@ -400,14 +410,14 @@ tw_get_cached_wikipedia_page_links <- function(title,
   cached_df <- db_result %>%
     dplyr::collect()
 
-  if (disconnect_db == TRUE) {
-    tw_disconnect_from_cache(
-      cache = TRUE,
-      cache_connection = db,
-      disconnect_db = disconnect_db,
-      language = language
-    )
-  }
+
+  tw_disconnect_from_cache(
+    cache = cache,
+    cache_connection = db,
+    disconnect_db = disconnect_db,
+    language = language
+  )
+
   cached_df
 }
 
@@ -483,14 +493,14 @@ tw_write_wikipedia_page_links_to_cache <- function(df,
     append = TRUE
   )
 
-  if (disconnect_db == TRUE) {
-    tw_disconnect_from_cache(
-      cache = TRUE,
-      cache_connection = db,
-      disconnect_db = disconnect_db,
-      language = language
-    )
-  }
+
+  tw_disconnect_from_cache(
+    cache = cache,
+    cache_connection = db,
+    disconnect_db = disconnect_db,
+    language = language
+  )
+
   invisible(df)
 }
 
@@ -537,12 +547,10 @@ tw_reset_wikipedia_page_links_cache <- function(language = tidywikidatar::tw_get
     usethis::ui_info(paste0("Wikipedia page links cache reset for language ", sQuote(language), " completed"))
   }
 
-  if (disconnect_db == TRUE) {
-    tw_disconnect_from_cache(
-      cache = TRUE,
-      cache_connection = db,
-      disconnect_db = disconnect_db,
-      language = language
-    )
-  }
+  tw_disconnect_from_cache(
+    cache = cache,
+    cache_connection = db,
+    disconnect_db = disconnect_db,
+    language = language
+  )
 }
